// LabWindows/CVI Example using VISA_Extended library for easier communication with a VISA instrument
// This source file and the VISA_Extended library can be easiliy imported into a Visual Studio C++ project
// This example is showing synchronization with the STB polling mechanism of:
// "SING" command, STB polling waits for the acquisition to finish
// "*TST?" query, STB polling waits for the selftest to finish and then reads the result of the selftest

#include <utility.h>
#include <ansi_c.h>
#include "VISA_Extended.h"

// error checking macro
#define CHECKERR(fCal) \
	status = (fCal);if (status < VI_SUCCESS) goto ERROR_HANDLING;

#define ARRAY_SIZES 200000

int main()
{
	ViSession scope;
	ViStatus status = VI_SUCCESS;
	char response[VISAEXT_BUFFER_SIZE];
	char instrumentErrors[16384];
	char generalerror[16384];

	//-----------------------------------------------------------
	//Initialization:
	//-----------------------------------------------------------
    SetBreakOnLibraryErrors (0); // Don't show Pupup messages with errors from libraries. Remove, if used in VS C++ project
    CHECKERR(VISAext_Open("TCPIP::192.168.1.101::INSTR", &scope)); //Adjust your instrument VISA resource string
	CHECKERR(VISAext_Query(scope, "*IDN?", response));
    printf("Device IDN string: %s", response);
	CHECKERR(VISAext_Write(scope, "*RST;*CLS;:SYST:DISP:UPD ON")); // Reset the instrument
	CHECKERR(VISAext_Write(scope, "*ESE 1")); // Event Status Enable OPC bit - one time setting after *RST
	CHECKERR(VISAext_ErrorChecking(scope, instrumentErrors)); // Error Checking
	//-----------------------------------------------------------
	// Settings all in one string:
	//---------------------------- -------------------------------
	CHECKERR(VISAext_Write(scope, "ACQ:POIN:AUTO RECL;:TIM:RANG 2.0;:ACQ:POIN 1002;:CHAN1:STAT ON;:TRIG1:MODE AUTO"));
	CHECKERR(VISAext_ErrorChecking(scope, instrumentErrors)); // Error Checking
	//-----------------------------------------------------------
	// Acquisition:
	//-----------------------------------------------------------
	// Sending SCPI command SING and using STB polling synchonization, timeout 6000 ms
    printf("\nAcquisition started...");
    CHECKERR(VISAext_WriteWithSTBpollSync(scope, "SING", 6000));
    printf("finished");
	CHECKERR(VISAext_ErrorChecking(scope, instrumentErrors)); // Error Checking
	
    // -----------------------------------------------------------
	// Selftest:
	// -----------------------------------------------------------
    // Synchronizing of a long-lasting command
    printf("\nSelftest started...");
    CHECKERR(VISAext_QueryWithSTBpollSync(scope, "*TST?", 120000, response));
    printf("finished");
	CHECKERR(VISAext_ErrorChecking(scope, instrumentErrors)); // Error Checking

ERROR_HANDLING:
	if (status == VI_SUCCESS)
	{
		; // all went OK
	}
	else if (status == VISAEXT_ERR_INSTRUMENT_STATUS)
	{
		//Instrument error occured
		printf("\nThe following instrument error(s) occured:\n%s", instrumentErrors);
	}
    else if (status == VISAEXT_ERR_STBPOLL_TIMEOUT)
    {
        //OPC polling timeout
        printf("\nOPC polling timeout occured");
    }
	else
	{
		//general error occured, try to get the string description from the status code
		VISAext_ErrorCodeDescription(scope, status, generalerror);
		printf("\nThe following error(s) occured:\n%s", generalerror);
	}

	printf("\n\nPress ENTER to continue...");
	(void)getchar();

	VISAext_Close(scope);

	return 0;
}
